function getMessage(url) {
  if (url.includes("photon-sol.tinyastro.io")) {
    if (url.includes("/memescope")) return "photon-memescope";
    if (url.includes("/lp/")) return "photon-token";
    if (url.includes("/discover")) return "photon-discover";
    if (url.includes("/trending")) return "photon-trending";
  } else if (url.includes("bullx.io")) {
    if (url.includes("neo.")) {
      if (url.includes("/neo-vision")) return "bullx-neo-vision";
      if (!url.split("neo.bullx.io/")[1]?.split("?")?.[0])
        return "bullx-neo-vision";
      if (url.includes("/terminal")) return "bullx-neo-token";
      if (url.includes("/explore")) return "bullx-neo-home";
      return 'bullx-neo'
    } else {
      if (url.includes("/pump-vision")) return "bullx-pump-vision";
      if (!url.split("bullx.io/")[1]?.split("?")?.[0]) return "bullx-home";
      if (url.includes("/terminal")) return "bullx-token";
    }
  } else if (url.includes("gmgn.ai")) {
    if (url.includes("/sol/token")) return "gmgn-token";
    const parsedUrl = new URL(url);
    const tab = parsedUrl.searchParams.get("tab");
    if (!tab) return "gmgn-empty-tab";
    if (tab === "home") return "gmgn-meme";
    if (tab === "new_pair") return "gmgn-home";
    if (tab === "new_creation") return "gmgn-home";
    if (tab === "completing") return "gmgn-home";
    if (tab === "soaring") return "gmgn-home";
    if (tab === "complete") return "gmgn-home";
    if (tab === "trending") return "gmgn-home";
    if (tab === "bluechip") return "gmgn-home";
  } else if (url.includes("dexscreener.com/solana/")) {
    return "dexscreener-token";
  } else if (url.includes("ape.pro")) {
    if (url.includes("/gems")) return "ape-gems";
    if (url.includes("/solana/")) return "ape-token";
    if (!url.split("ape.pro/")[1]?.split("?")?.[0]) return "ape-home";
  }
  return;
}

chrome.tabs.onUpdated.addListener((tabId, changeInfo) => {
  if (!changeInfo.url) return;
  const message = getMessage(changeInfo.url);
  if (message) {
    const auth = checkAuthentication();
    chrome.tabs.sendMessage(tabId, {
      message,
      event: "onUpdated",
      url: changeInfo.url,
      auth,
    });
  }
});

chrome.tabs.onActivated.addListener((details) => {
  const { tabId } = details;
  chrome.tabs.get(tabId, (tab) => {
    const { url } = tab;
    const message = getMessage(url);
    if (message) {
      const auth = checkAuthentication();
      chrome.tabs.sendMessage(tabId, {
        message,
        event: "onActivated",
        url,
        auth,
      });
    }
  });
});

chrome.webNavigation.onCompleted.addListener((details) => {
  if (!details.url) return;
  const message = getMessage(details.url);
  if (message) {
    const auth = checkAuthentication();
    chrome.tabs.sendMessage(details.tabId, {
      message,
      event: "onCompleted",
      url: details.url,
      auth,
    });
  }
});

chrome.runtime.onMessage.addListener(function (request, _sender, sendResponse) {
  if (request.message === "logout") {
    chrome.storage.local.remove(
      ["bloom.token", "bloom.expires_at"],
      function () {
        chrome.action.setPopup({ popup: "src/public/landing.html" });
        sendResponse({ success: true });
      },
    );
    return true;
  } else if (request.message === "openTab") {
    chrome.tabs.create({
      url: request.url,
    });
    return true;
  } else if (request.message === "checkAuth") {
    const result = checkAuthentication();
    sendResponse(result);
    return true;
  }
});

function checkAuthentication() {
  chrome.storage.local.get(
    ["bloom.token", "bloom.expires_at"],
    function (result) {
      const token = result["bloom.token"];
      const expiresAt = result["bloom.expires_at"];

      if (token && expiresAt) {
        const currentTime = Date.now();
        if (currentTime < expiresAt) {
          chrome.action.setPopup({ popup: "src/public/popup.html" });
          return result;
        } else {
          chrome.storage.local.remove(
            ["bloom.token", "bloom.expires_at"],
            function () {
              chrome.action.setPopup({ popup: "src/public/landing.html" });
            },
          );
          return false;
        }
      } else {
        chrome.action.setPopup({ popup: "src/public/landing.html" });
        return false;
      }
    },
  );
}

checkAuthentication();
