function insertAfter(referenceNode, newNode) {
  referenceNode.parentNode.insertBefore(newNode, referenceNode.nextSibling);
}

function insertBefore(referenceNode, newNode) {
  referenceNode.parentNode.insertBefore(newNode, referenceNode);
}

chrome.runtime.onMessage.addListener(async function (request) {
  if (request.message === "ape-gems") {
    const container = await findGemsContainer();
    if (container) {
      addGemsQTButtons();
      const observer = new MutationObserver(() => addGemsQTButtons());
      observer.observe(container, { childList: true, subtree: true });
    }
  }
  if (request.message === "ape-token") {
    const buySellContainer = await findBuySellContainer();
    if (buySellContainer) {
      addCustomBuyButton();
      const observer = new MutationObserver((mutations) => {
        for (const m of mutations) {
          if (
            Array.from(m.addedNodes)
              .concat(m.removedNodes)
              .find((nodes) =>
                Array.from(nodes).find((n) =>
                  n?.innerText?.includes("undergoing a liquidity migration"),
                ),
              )
          ) {
            addCustomBuyButton();
          }
        }
      });
      observer.observe(buySellContainer, { childList: true });
    }
  }
  if (request.message === "ape-home") {
    const container = await findMain();
    if (container) {
      addNewTokensButtons();
      const observer = new MutationObserver(() => addNewTokensButtons());
      observer.observe(container, { childList: true });
    }
  }
});

async function findMain(timeout = 5000) {
  for (let i = 0; i < timeout / 500; i++) {
    const main = document.querySelector("div.grid.w-full");
    if (main) return main;
    await new Promise((r) => setTimeout(r, 500));
  }
  return null;
}

async function findGemsContainer(timeout = 5000) {
  for (let i = 0; i < timeout / 500; i++) {
    const container = document.querySelector("div.grid-cols-1");
    if (container) return container;
    await new Promise((r) => setTimeout(r, 500));
  }
  return null;
}

async function findBuySellContainer(timeout = 5000) {
  for (let i = 0; i < timeout / 500; i++) {
    const container = document.querySelector("div.w-full.overflow-hidden");
    if (container) {
      return container;
    }
    await new Promise((r) => setTimeout(r, 500));
  }
  return null;
}

function addCustomBuyButton() {
  try {
    const isMigrating = Array.from(document.querySelectorAll("span.flex")).find(
      (span) => span.innerText === "Migrating...",
    );

    const existingBuyButton = document.querySelector(".bloom-buy-qt-btn");
    const existingSnipeButton = document.querySelector(".bloom-snipe-qt-btn");

    if (existingBuyButton && isMigrating) {
      existingBuyButton.remove();
    } else if (existingSnipeButton && !isMigrating) {
      existingSnipeButton.remove();
    } else if (existingBuyButton || existingSnipeButton) {
      return;
    }

    const actionArea = document.querySelector("button.w-full.font-semibold");
    if (!actionArea) return;

    if (isMigrating) {
      const bloomButton = document.createElement("button");
      bloomButton.innerHTML = "🌸 Snipe with Bloom";
      bloomButton.type = "button";
      bloomButton.classList.add("bloom-snipe-qt-btn");
      Array.from(actionArea.classList)
        .filter((c) => !c.includes("disabled") && !c.includes("opacity"))
        .forEach((c) => bloomButton.classList.add(c));
      bloomButton.style.border = "1px solid #EEA7ED";
      bloomButton.style.background = "none";
      bloomButton.style.setProperty("--un-ring-color", "#EEA7ED");

      bloomButton.onclick = function () {
        const tokenMint = window.location.href.split("/").pop();

        chrome.runtime.sendMessage({
          message: "openTab",
          url: `https://t.me/BloomSolana_bot?start=ref_QT_sniper_${tokenMint}`,
        });
      };

      insertAfter(actionArea, bloomButton);
    } else {
      const bloomButton = document.createElement("button");
      bloomButton.innerHTML = "🌸 Bloom";
      bloomButton.type = "button";
      bloomButton.classList.add("bloom-buy-qt-btn");
      Array.from(actionArea.classList)
        .filter((c) => !c.includes("disabled") && !c.includes("opacity"))
        .forEach((c) => bloomButton.classList.add(c));
      bloomButton.style.border = "1px solid #EEA7ED";
      bloomButton.style.background = "none";
      bloomButton.style.setProperty("--un-ring-color", "#EEA7ED");

      bloomButton.onclick = function () {
        const tokenMint = window.location.href.split("/").pop();

        chrome.runtime.sendMessage({
          message: "openTab",
          url: `https://t.me/BloomSolana_bot?start=ref_QT_ca_${tokenMint}`,
        });
      };

      insertAfter(actionArea, bloomButton);
    }
  } catch (error) {
    console.error("Failed to add Custom Buy button:", error);
  }
}

function addGemsQTButtons() {
  try {
    const cards = Array.from(document.querySelectorAll("div.h-20"));
    cards.forEach((card) => {
      const isMigrating = Array.from(card.querySelectorAll("div.text-xs")).some(
        (div) => div.textContent === "Migrating",
      );

      const existingBuyButton = card.querySelector(".bloom-buy-qt-btn");
      const existingSnipeButton = card.querySelector(".bloom-snipe-qt-btn");

      if (existingBuyButton || existingSnipeButton) {
        if (isMigrating && existingBuyButton) {
          existingBuyButton.remove();
        } else if (!isMigrating && existingSnipeButton) {
          existingSnipeButton.remove();
        } else {
          return;
        }
      }

      const poolLink = card.querySelector('a[preload="false"]');
      if (!poolLink) return;

      const actionArea = card.querySelector("button.text-xs");
      if (!actionArea) return;

      const buttonClass = isMigrating
        ? "bloom-snipe-qt-btn"
        : "bloom-buy-qt-btn";
      const buttonText = isMigrating ? "🌸 Snipe" : "🌸 Buy";

      const bloomButton = document.createElement("button");

      bloomButton.innerHTML = buttonText;
      bloomButton.type = "button";
      bloomButton.classList.add(
        buttonClass,
        "rounded",
        "px-2",
        "ml-auto",
        "text-xs",
        "h-8",
      );

      bloomButton.style.border = "1px solid #EEA7ED";
      bloomButton.style.zIndex = "1000";

      const tokenMint = poolLink.href.split("/").pop();

      bloomButton.onclick = function (event) {
        event.preventDefault();
        event.stopPropagation();
        chrome.runtime.sendMessage({
          message: "openTab",
          url: `https://t.me/BloomSolana_bot?start=ref_QT_${
            isMigrating ? "sniper" : "ca"
          }_${tokenMint}`,
        });
      };

      insertBefore(actionArea, bloomButton);
    });
  } catch (error) {
    console.log("Failed to add Bloom button:", error);
  }
}

function addNewTokensButtons() {
  try {
    const cards = Array.from(document.querySelectorAll("div.group.contents"));
    cards.forEach((card) => {
      const existingBuyButton = card.querySelector(".bloom-buy-qt-btn");
      if (existingBuyButton) {
        return;
      }

      const poolLink = card.querySelector('a[preload="false"]');
      if (!poolLink) return;

      const actionArea = card.querySelector("button.text-xs");
      if (!actionArea) return;

      const buttonClass = "bloom-buy-qt-btn";
      const buttonText = "🌸 Buy";

      const bloomButton = document.createElement("button");

      bloomButton.innerHTML = buttonText;
      bloomButton.type = "button";
      bloomButton.classList.add(
        buttonClass,
        "rounded",
        "px-2",
        "ml-auto",
        "mr-2",
        "text-xs",
        "h-8",
      );

      bloomButton.style.border = "1px solid #EEA7ED";
      bloomButton.style.zIndex = "1000";

      const tokenMint = poolLink.href.split("/").pop();

      bloomButton.onclick = function (event) {
        event.preventDefault();
        event.stopPropagation();
        chrome.runtime.sendMessage({
          message: "openTab",
          url: `https://t.me/BloomSolana_bot?start=ref_QT_ca_${tokenMint}`,
        });
      };

      insertBefore(actionArea, bloomButton);
    });
  } catch (error) {
    console.log("Failed to add Bloom button:", error);
  }
}
