const activeObservers = {};
const buyButtonClassName = "b" + Math.random().toString(36).substring(2, 10);
const sellButtonClassName = "s" + Math.random().toString(36).substring(2, 10);
const snipeButtonClassName = "n" + Math.random().toString(36).substring(2, 10);
const toastClassName = "t" + Math.random().toString(36).substring(2, 10);
const oneClickButtonClassName = "o" + Math.random().toString(36).substring(2, 10);
const walletManagerButtonClassName = "w" + Math.random().toString(36).substring(2, 10);
const topBarClassName = "tb" + Math.random().toString(36).substring(2, 10);

// LISTENER

let currentPage, quickBuyAmount;
chrome.runtime.onMessage.addListener(async function (request) {
  quickBuyAmount =
    (await chrome.storage.local.get("bloom.quickBuyAmount"))?.[
    "bloom.quickBuyAmount"
    ] || 0;
  if (request.message === "bullx-neo") {
    const token =
      (await chrome.storage.local.get("bloom.token"))?.["bloom.token"] || "";
    if (!token && request.event === "onCompleted") {
      showToast(
        "Log in to the Bloom extension to enhance your experience!",
        "top-center",
      );
    }

    const observers = activeObservers["other"] || [];
    observers.forEach((observer) => observer.disconnect());
    currentPage = request.message;
    handleWalletManager('other');
  }
  if (request.message === "bullx-neo-vision") {
    const token =
      (await chrome.storage.local.get("bloom.token"))?.["bloom.token"] || "";
    if (!token && request.event === "onCompleted") {
      showToast(
        "Log in to the Bloom extension to enhance your experience!",
        "top-center",
      );
    }
    const observers = activeObservers["neo-vision"] || [];
    observers.forEach((observer) => observer.disconnect());
    currentPage = request.message;
    handleWalletManager('neo-vision');
    handleNeoVision();
  }
  if (request.message === "bullx-neo-token") {
    const token =
      (await chrome.storage.local.get("bloom.token"))?.["bloom.token"] || "";
    if (!token && request.event === "onCompleted") {
      showToast(
        "Log in to the Bloom extension to enhance your experience!",
        "top-center",
      );
    }
    const observers = activeObservers["token"] || [];
    observers.forEach((observer) => observer.disconnect());
    currentPage = request.message;
    handleWalletManager('token');
    handleToken();
    handleOneClick();
  }
  if (request.message === "bullx-neo-home") {
    const token =
      (await chrome.storage.local.get("bloom.token"))?.["bloom.token"] || "";
    if (!token && request.event === "onCompleted") {
      showToast(
        "Log in to the Bloom extension to enhance your experience!",
        "top-center",
      );
    }
    const observers = activeObservers["home"] || [];
    observers.forEach((observer) => observer.disconnect());
    currentPage = request.message;
    const main = await findMain();
    if (main) {
      handleWalletManager('home');
      handleBullxHome();
    }
  }
  if (request.message === "quickBuyAmount") {
    quickBuyAmount =
      (await chrome.storage.local.get("bloom.quickBuyAmount"))?.[
      "bloom.quickBuyAmount"
      ] || 0;
    if (currentPage === "bullx-neo-home") {
      const cards = Array.from(
        document.querySelectorAll("a[href*='/terminal']"),
      );
      handleWalletManager('home', true);
      cards.forEach((card) =>
        addDiscoverTrendingBuyButton(card, quickBuyAmount),
      );
    } else if (currentPage === "bullx-neo-vision") {
      const cards = Array.from(document.querySelectorAll("div.neo-card")).concat(Array.from(document.querySelectorAll("div.pump-card")));
      handleWalletManager('neo-vision', true);
      cards.forEach((card) => addNeoVisionButtons(card, quickBuyAmount, true));
    } else if (currentPage === "bullx-neo-token") {
      handleWalletManager('token', true);
    } else {
      handleWalletManager('other', true);
    }
  }
});

// UTILS

function insertAfter(referenceNode, newNode) {
  referenceNode.parentNode.insertBefore(newNode, referenceNode.nextSibling);
}

function insertBefore(referenceNode, newNode) {
  referenceNode.parentNode.insertBefore(newNode, referenceNode);
}

// ORDERS

async function createOrder(address, isSnipe, amount, authToken, side) {
  try {
    const res = await fetch(
      `https://extension.bloombot.app/${isSnipe ? "snipe" : "swap"}`,
      {
        method: "POST",
        headers: {
          "Content-Type": "application/json",
        },
        body: JSON.stringify(
          isSnipe
            ? {
              addr: address,
              auth: authToken,
            }
            : {
              addr: address,
              isPool: false,
              amt: parseFloat(amount).toString(),
              auth: authToken,
              side,
            },
        ),
      },
    );

    return res.json();
  } catch (error) {
    console.error("Failed to create order:", error);
    return null;
  }
}

// TOAST

function showToast(message, position = "bottom-right") {
  const previousToasts = document.querySelectorAll(`.${toastClassName}`);
  previousToasts.forEach((toast) => toast.remove());

  const toast = document.createElement("div");
  toast.style.position = "fixed";
  if (position === "bottom-right") {
    toast.style.bottom = "20px";
    toast.style.right = "20px";
  } else if (position === "top-center") {
    toast.style.top = "20px";
    toast.style.left = "50%";
    toast.style.transform = "translateX(-50%)";
  }
  toast.style.backgroundColor = "#0d0d10";
  toast.style.color = "#EEA7ED";
  toast.style.padding = "12px 24px";
  toast.style.borderRadius = "5px";
  toast.style.zIndex = "10000";
  toast.style.fontSize = "14px";
  toast.style.fontWeight = "600";
  toast.style.border = "1px solid #EEA7ED";
  toast.textContent = "🌸 " + message;
  toast.classList.add(toastClassName);

  document.body.appendChild(toast);

  setTimeout(() => {
    toast.style.transition = "opacity 0.5s ease-out";
    toast.style.opacity = "0";
    setTimeout(() => toast.remove(), 500);
  }, 3000);
}

// FINDERS

async function findNeoVisionContainer(timeout = 5000) {
  for (let i = 0; i < timeout / 500; i++) {
    const containers = Array.from(
      document.querySelectorAll(".w-full.flex-col.min-h-0"),
    );
    if (containers.length > 0) return containers;
    await new Promise((r) => setTimeout(r, 500));
  }
  return null;
}

async function findMain(timeout = 5000) {
  for (let i = 0; i < timeout / 500; i++) {
    const main = document.querySelector(".b-table-container");
    if (main) return main;
    await new Promise((r) => setTimeout(r, 500));
  }
  return null;
}

async function findTableContainer(timeout = 5000) {
  for (let i = 0; i < timeout / 500; i++) {
    const container = document.querySelector(".b-table-body");
    if (container) return container;
    await new Promise((r) => setTimeout(r, 500));
  }
  return null;
}

async function findTopBar(timeout = 5000) {
  for (let i = 0; i < timeout / 500; i++) {
    const topBar = document.querySelector("div.w-full.no-scrollbar.border-y");
    if (topBar) {
      return topBar;
    }
    await new Promise((r) => setTimeout(r, 500));
  }
  return null;
}

async function findBuySellContainer(timeout = 5000) {
  for (let i = 0; i < timeout / 500; i++) {
    const container = document.querySelector("div.ant-drawer-content-wrapper");
    if (container) {
      return container;
    }
    await new Promise((r) => setTimeout(r, 500));
  }
  return null;
}

async function findOneClickContainer(timeout = 5000) {
  for (let i = 0; i < timeout / 500; i++) {
    const container = document.querySelector("div.one-click-container");
    if (container) return container;
    await new Promise((r) => setTimeout(r, 500));
  }
  return null;
}

async function findWalletManagerContainer(timeout = 5000) {
  for (let i = 0; i < timeout / 500; i++) {
    const container = document.querySelector("div.wallet-manager");
    if (container) return container;
    await new Promise((r) => setTimeout(r, 500));
  }
  return null;
}

// HANDLERS

async function handleOneClick() {
  const container = await findOneClickContainer();
  if (container) {
    addOneClickButtons(container);
  }

  const observer = new MutationObserver((m) => {
    m.forEach((mutation) => {
      mutation.addedNodes.forEach((node) => {
        if (node.nodeName?.toLowerCase() === "div" && node.classList.contains("one-click-container")) {
          addOneClickButtons(node);
        } else if (node.nodeName?.toLowerCase() === "div" && node.childNodes.length > 0) {
          const oneClickContainer = node.querySelector("div.one-click-container");
          if (oneClickContainer) {
            addOneClickButtons(oneClickContainer);
          }
        }
      });
    });
  });

  activeObservers["token"] ? activeObservers["token"].push(observer) : (activeObservers["token"] = [observer]);
  observer.observe(document.body, { childList: true, subtree: true });
}

async function handleWalletManager(page, reload = false) {
  quickBuyAmount =
    (await chrome.storage.local.get("bloom.quickBuyAmount"))?.[
    "bloom.quickBuyAmount"
    ] || 0;
  const container = await findWalletManagerContainer();
  if (container) {
    handleWalletManagerContainer(container, reload);
  }

  const observer = new MutationObserver((m) => {
    m.forEach((mutation) => {
      mutation.addedNodes.forEach((node) => {
        if (node.nodeName?.toLowerCase() === "div" && node.classList.contains("wallet-manager")) {
          handleWalletManagerContainer(node, reload);
        } else if (node.nodeName?.toLowerCase() === "div" && node.childNodes.length > 0) {
          const walletManagerContainer = node.querySelector("div.wallet-manager");
          if (walletManagerContainer) {
            handleWalletManagerContainer(walletManagerContainer, reload);
          }
        }
      });
    });
  });

  activeObservers[page] ? activeObservers[page].push(observer) : (activeObservers[page] = [observer]);
  observer.observe(document.body, { childList: true, subtree: true });
}

async function handleWalletManagerContainer(container, reload = false) {
  const allRows = Array.from(container.querySelectorAll('div.trades-row'));
  allRows.forEach((row) => addWalletManagerButton(row, reload));

  const modal = container.closest('div.ant-modal');
  if (!modal);

  modal.style.width = '880px';

  const observer = new MutationObserver((m) => {
    m.forEach((mutation) => {
      mutation.addedNodes.forEach((node) => {
        if (node.classList?.contains('trades-row')) {
          addWalletManagerButton(node, reload);
        } else if (node.childNodes.length > 0) {
          const rows = Array.from(node.querySelectorAll('div.trades-row'));
          rows.forEach((row) => addWalletManagerButton(row, reload));
        }
      });
    });
  });

  observer.observe(container, { childList: true, subtree: true });
  activeObservers["token"] ? activeObservers["token"].push(observer) : (activeObservers["token"] = [observer]);
}

async function handleNeoVision(idx) {
  const containers = await findNeoVisionContainer();
  if (idx && containers[idx]) {
    handleNeoVisionContainer(containers[idx], idx);
  } else if (containers && containers.length > 0) {
    containers.forEach((container, idx) =>
      handleNeoVisionContainer(container, idx),
    );
  }
}

async function handleNeoVisionContainer(container, idx) {
  quickBuyAmount =
    (await chrome.storage.local.get("bloom.quickBuyAmount"))?.[
    "bloom.quickBuyAmount"
    ] || 0;
  const cards = Array.from(container.querySelectorAll("div.neo-card")).concat(Array.from(container.querySelectorAll("div.pump-card")));
  cards.forEach((card) => addNeoVisionButtons(card, quickBuyAmount));
  const observer = new MutationObserver((mutations) => {
    mutations.forEach((mutation) => {
      mutation.addedNodes.forEach((node) => {
        if (
          node.nodeName &&
          node.nodeName.toLowerCase() === "div" &&
          (node.classList.contains("neo-card") || node.classList.contains("pump-card"))
        ) {
          addNeoVisionButtons(node, quickBuyAmount);
        } else if (node.nodeName && node.nodeName.toLowerCase() === "div") {
          const hasMigration = Array.from(node.childNodes).find(
            (child) =>
              child.nodeName?.toLowerCase() === "span" &&
              child.textContent === "Migrating...",
          );
          if (hasMigration) {
            const card = node.closest(".neo-card") || node.closest(".pump-card");
            if (card) {
              addNeoVisionButtons(card, quickBuyAmount);
            }
          }
        }
      });
    });
  });

  observer.observe(container.firstElementChild, {
    childList: true,
    subtree: true,
  });
  activeObservers["neo-vision"]
    ? activeObservers["neo-vision"].push(observer)
    : (activeObservers["neo-vision"] = [observer]);

  const containerParent = container.parentElement;
  if (containerParent) {
    const parentObserver = new MutationObserver(() => {
      observer.disconnect();
      parentObserver.disconnect();
      return handleNeoVision(idx);
    });

    parentObserver.observe(containerParent, { childList: true });
    activeObservers["neo-vision"]
      ? activeObservers["neo-vision"].push(parentObserver)
      : (activeObservers["neo-vision"] = [parentObserver]);
  }
}

function handleNeoButton(neoButton, isSell = false) {
  const observer = new MutationObserver((m) => {
    const newNeoButton = isSell ? addSellButton() : addBuyButton();
    observer.disconnect();
    if (newNeoButton) {
      handleNeoButton(newNeoButton, isSell);
    }
  });

  observer.observe(neoButton, { characterData: true, subtree: true });
  activeObservers["token"]
    ? activeObservers["token"].push(observer)
    : (activeObservers["token"] = [observer]);
  return observer;
}

async function handleToken() {
  try {
    const topBar = await findTopBar();
    const buySellContainer = await findBuySellContainer();
    if (!topBar || !buySellContainer) return;
    let migrationContainer = document.querySelector("div.buy-sell-migrating");

    const existingBuyButton = document.querySelector(`.${buyButtonClassName}`);
    const existingSnipeButton = document.querySelector(
      `.${snipeButtonClassName}`,
    );

    if (existingBuyButton) existingBuyButton.remove();
    if (existingSnipeButton) existingSnipeButton.remove();

    if (migrationContainer) {
      addSnipingButton(migrationContainer);
    }

    addTopBarButton();
    handleOrderForm();

    const observer = new MutationObserver(() => {
      const newMigrationContainer = document.querySelector(
        "div.buy-sell-migrating",
      );
      if (Boolean(newMigrationContainer) !== Boolean(migrationContainer)) {
        migrationContainer = newMigrationContainer;
        if (migrationContainer) {
          addSnipingButton(migrationContainer);
        } else {
          const snipeButton = document.querySelector(
            `.${snipeButtonClassName}`,
          );
          if (snipeButton) snipeButton.remove();
        }
      }
    });

    observer.observe(buySellContainer, { childList: true, subtree: true });
    activeObservers["token"]
      ? activeObservers["token"].push(observer)
      : (activeObservers["token"] = [observer]);
  } catch (error) {
    console.error("Failed to add Custom Buy button:", error);
  }
}

function handleBuyDiv() {
  const buyDiv = document.querySelector("div.market-buy");
  if (!buyDiv) return;
  let neoBuyButton = addBuyButton();
  let neoButtonObserver = neoBuyButton ? handleNeoButton(neoBuyButton) : null;
  const buyDivObserver = new MutationObserver((m) => {
    m.forEach((mutation) => {
      mutation.addedNodes.forEach((node) => {
        if (
          node.nodeName.toLowerCase() === "div" &&
          (node?.classList?.contains("market-order") ||
            node?.classList?.contains("ant-tabs-tabpane-active"))
        ) {
          if (neoButtonObserver) neoButtonObserver.disconnect();
          neoBuyButton = addBuyButton();
          if (neoBuyButton) {
            neoButtonObserver = handleNeoButton(neoBuyButton);
          }
        }
      });
    });
  });

  buyDivObserver.observe(buyDiv, { childList: true, subtree: true });
  activeObservers["token"]
    ? activeObservers["token"].push(buyDivObserver)
    : (activeObservers["token"] = [buyDivObserver]);
}

function handleSellDiv() {
  const sellDiv = document.querySelector("div.market-sell");
  if (!sellDiv) return;
  let neoSellButton = addSellButton();
  let neoSellButtonObserver = neoSellButton
    ? handleNeoButton(neoSellButton, true)
    : null;
  const sellDivObserver = new MutationObserver((m) => {
    m.forEach((mutation) => {
      mutation.addedNodes.forEach((node) => {
        if (
          node.nodeName.toLowerCase() === "div" &&
          (node?.classList?.contains("market-order") ||
            node?.classList?.contains("ant-tabs-tabpane-active"))
        ) {
          if (neoSellButtonObserver) neoSellButtonObserver.disconnect();
          neoSellButton = addSellButton();
          if (neoSellButton) {
            neoSellButtonObserver = handleNeoButton(neoSellButton, true);
          }
        }
      });
    });
  });

  sellDivObserver.observe(sellDiv, { childList: true, subtree: true });
  activeObservers["token"]
    ? activeObservers["token"].push(sellDivObserver)
    : (activeObservers["token"] = [sellDivObserver]);
}

function handleOrderForm() {
  const customTab = document.querySelector("div.custom-tab");
  if (!customTab) return;
  const orderForm = customTab.parentElement;
  if (!orderForm) return;
  let buyObserver = handleBuyDiv();
  let sellObserver = handleSellDiv();
  const orderFormObserver = new MutationObserver((m) => {
    m.forEach((mutation) => {
      mutation.addedNodes.forEach((node) => {
        if (
          node.nodeName.toLowerCase() === "div" &&
          node?.classList?.contains("ant-tabs-tabpane-active") &&
          node?.firstElementChild?.classList?.contains("market-buy")
        ) {
          if (buyObserver) buyObserver.disconnect();
          buyObserver = handleBuyDiv();
        }
        if (
          node.nodeName.toLowerCase() === "div" &&
          node?.classList?.contains("ant-tabs-tabpane-active") &&
          node?.firstElementChild?.classList?.contains("market-sell")
        ) {
          if (sellObserver) sellObserver.disconnect();
          sellObserver = handleSellDiv();
        }
      });
      mutation.removedNodes.forEach((node) => {
        if (
          node.nodeName.toLowerCase() === "div" &&
          node?.classList?.contains("ant-tabs-tabpane-active") &&
          node?.firstElementChild?.classList?.contains("market-buy")
        ) {
          if (buyObserver) buyObserver.disconnect();
          buyObserver = null;
        }
        if (
          node.nodeName.toLowerCase() === "div" &&
          node?.classList?.contains("ant-tabs-tabpane-active") &&
          node?.firstElementChild?.classList?.contains("market-sell")
        ) {
          if (sellObserver) sellObserver.disconnect();
          sellObserver = null;
        }
      });
    });
  });

  orderFormObserver.observe(orderForm.parentElement, {
    childList: true,
    subtree: true,
  });
  activeObservers["token"]
    ? activeObservers["token"].push(orderFormObserver)
    : (activeObservers["token"] = [orderFormObserver]);
}

async function handleBullxHome() {
  const container = await findTableContainer();
  if (container) {
    quickBuyAmount =
      (await chrome.storage.local.get("bloom.quickBuyAmount"))?.[
      "bloom.quickBuyAmount"
      ] || 0;
    const cards = Array.from(container.querySelectorAll("a.b-table-row"));
    cards.forEach((card) => addDiscoverTrendingBuyButton(card, quickBuyAmount));
    const observer = new MutationObserver((m) => {
      const addedRows = Array.from(m)
        .map((m) => Array.from(m.addedNodes))
        .flat()
        .filter((r) => r.classList?.contains("b-table-row"));
      addedRows.forEach((row) =>
        addDiscoverTrendingBuyButton(row, quickBuyAmount),
      );
    });

    observer.observe(container, { childList: true, subtree: true });
    activeObservers["home"]
      ? activeObservers["home"].push(observer)
      : (activeObservers["home"] = [observer]);
  }
}

// INJECTORS

function addSnipingButton(migrationContainer) {
  const migrationText = Array.from(
    migrationContainer.querySelectorAll("p"),
  ).filter((p) => p.textContent.includes("may take a few minutes"))?.[0];
  if (!migrationText) return;

  const bloomButton = document.createElement("button");
  bloomButton.innerHTML = "🌸 Snipe with Bloom";
  bloomButton.type = "button";
  bloomButton.classList.add(snipeButtonClassName, "ant-btn", "ant-btn-text");
  bloomButton.style.marginTop = "6px";
  bloomButton.style.marginBottom = "-24px";
  bloomButton.style.border = "1px solid #EEA7ED";

  bloomButton.onclick = async function () {
    const url = new URL(window.location.href);
    const tokenMint = url.searchParams.get("address");
    if (!tokenMint) return showToast("Token not found");

    const token =
      (await chrome.storage.local.get("bloom.token"))?.["bloom.token"] || "";
    if (!token) {
      return chrome.runtime.sendMessage({
        message: "openTab",
        url: `https://t.me/BloomSolana_bot?start=ref_QT_sniper_${tokenMint}`,
      });
    }

    const order = await createOrder(tokenMint, true, 0, token, "buy");
    if (order?.status === "success") {
      showToast("Order placed successfully!");
    } else {
      showToast("Failed placing order!");
    }
  };

  insertBefore(migrationText, bloomButton);
}

function addBuyButton() {
  const neoBuyButton = document.querySelector(".ant-btn-primary");
  if (!neoBuyButton) return;
  if (neoBuyButton.innerHTML.includes("Add Funds")) return;
  const parentDiv = neoBuyButton.parentElement;
  if (!parentDiv) return neoBuyButton;
  const buyDiv = document.querySelector("div.market-buy");
  if (!buyDiv) return neoBuyButton;
  const activeTab = buyDiv.querySelector("div.ant-tabs-tab-active");
  const currentBloomButtons = document.querySelectorAll(
    `.${buyButtonClassName}`,
  );

  currentBloomButtons.forEach((button) => button.remove());

  if (!activeTab || activeTab.getAttribute("data-node-key") === "DCA") {
    return neoBuyButton;
  }

  parentDiv.style.flexDirection = "column";

  const bloomButton = neoBuyButton.cloneNode(true);
  bloomButton.classList.add(buyButtonClassName);
  bloomButton.style.marginTop = "6px";
  bloomButton.style.setProperty("border", "1px solid #EEA7ED", "important");
  bloomButton.style.setProperty("background-color", "#EEA7EDB3", "important");
  const firstSpan = bloomButton.querySelector("span");
  if (firstSpan) {
    firstSpan.textContent = "🌸 " + firstSpan.textContent;
  }

  bloomButton.onclick = async function (event) {
    event.preventDefault();
    event.stopPropagation();

    const url = new URL(window.location.href);
    const tokenMint = url.searchParams.get("address");
    if (!tokenMint) return showToast("Token not found");

    const token =
      (await chrome.storage.local.get("bloom.token"))?.["bloom.token"] || "";
    if (!token) {
      return chrome.runtime.sendMessage({
        message: "openTab",
        url: `https://t.me/BloomSolana_bot?start=ref_QT_ca_${tokenMint}`,
      });
    }

    const tabName = activeTab.getAttribute("data-node-key");
    if (tabName !== "MARKET") {
      return showToast("Limit orders are not supported yet!");
    }

    const amtInput = buyDiv.querySelector('input[type="number"]');
    if (!amtInput) return showToast("Invalid amount!");
    const amount = amtInput.value;
    if (!amount || isNaN(parseFloat(amount)))
      return showToast("Invalid amount!");

    const order = await createOrder(tokenMint, false, amount, token, "buy");

    if (order?.status === "success") {
      showToast("Order placed successfully!");
    } else {
      showToast("Failed placing order!");
    }
  };

  insertAfter(neoBuyButton, bloomButton);
  return neoBuyButton;
}

function addSellButton() {
  try {
    const neoSellButton = document.querySelector(".ant-btn-primary");
    if (!neoSellButton) return;
    const parentDiv = neoSellButton.parentElement;
    if (!parentDiv) return;
    const sellDiv = document.querySelector("div.market-sell");
    if (!sellDiv) return;
    const activeTab = sellDiv.querySelector("div.ant-tabs-tab-active");
    const currentBloomButtons = document.querySelectorAll(
      `.${sellButtonClassName}`,
    );
    currentBloomButtons.forEach((button) => button.remove());

    if (!activeTab || activeTab.getAttribute("data-node-key") === "DCA") {
      return neoSellButton;
    }

    parentDiv.style.flexDirection = "column";

    const bloomButton = neoSellButton.cloneNode(true);
    bloomButton.classList.add(sellButtonClassName);
    bloomButton.style.marginTop = "6px";
    bloomButton.style.setProperty("border", "1px solid #EEA7ED", "important");
    bloomButton.style.setProperty("background-color", "#EEA7EDB3", "important");
    const firstSpan = bloomButton.querySelector("span");
    if (firstSpan) {
      firstSpan.textContent = "🌸 Sell";
    }

    bloomButton.onclick = async function (event) {
      event.preventDefault();
      event.stopPropagation();

      const url = new URL(window.location.href);
      const tokenMint = url.searchParams.get("address");
      if (!tokenMint) return showToast("Token not found");

      const token =
        (await chrome.storage.local.get("bloom.token"))?.["bloom.token"] || "";
      if (!token) {
        return chrome.runtime.sendMessage({
          message: "openTab",
          url: `https://t.me/BloomSolana_bot?start=ref_QT_ca_${tokenMint}`,
        });
      }

      const tabName = activeTab.getAttribute("data-node-key");
      if (tabName !== "MARKET") {
        return showToast("Limit orders are not supported yet!");
      }

      const amtContainer = sellDiv.querySelector(
        "div.ant-input-number-affix-wrapper.rounded",
      );
      if (!amtContainer) return showToast("Invalid amount!");
      const amtInput = amtContainer.querySelector('input[type="number"]');
      if (!amtInput) return showToast("Invalid amount!");
      const amount = amtInput.value;
      if (!amount || isNaN(parseFloat(amount)))
        return showToast("Invalid amount!");

      const order = await createOrder(tokenMint, false, amount, token, "sell");

      if (order?.status === "success") {
        showToast("Order placed successfully!");
      } else {
        showToast("Failed placing order!");
      }
    };

    const lastDiv = bloomButton.querySelector("div:last-child");
    const amtContainer = sellDiv.querySelector(
      "div.ant-input-number-affix-wrapper.rounded",
    );
    const amtInput = amtContainer?.querySelector('input[type="number"]');
    if (amtInput) {
      if (bloomButton.disabled && amtInput.value > 0) {
        bloomButton.disabled = false;
      }
      lastDiv.innerHTML = `${amtInput.value || 0} %`;

      const observer = new MutationObserver(() => {
        if (bloomButton.disabled && amtInput.value > 0) {
          bloomButton.disabled = false;
        }
        lastDiv.innerHTML = `${amtInput.value || 0} %`;
      });
      observer.observe(amtInput, {
        attributes: true,
        attributeFilter: ["value"],
      });

      amtInput.addEventListener("input", () => {
        if (bloomButton.disabled && amtInput.value > 0) {
          bloomButton.disabled = false;
        }
        lastDiv.innerHTML = `${amtInput.value || 0} %`;
      });
    }

    insertAfter(neoSellButton, bloomButton);
    return neoSellButton;
  } catch (error) {
    console.error("Failed to add Sell button:", error);
  }
}

function addNeoVisionButtons(card, quickBuyAmount) {
  try {
    const migrationContainer = Array.from(card.querySelectorAll("span")).find(
      (span) => span.textContent === "Migrating...",
    );

    const existingBuyButton = card.querySelector(`.${buyButtonClassName}`);
    const existingSnipeButton = card.querySelector(`.${snipeButtonClassName}`);

    if (existingBuyButton) existingBuyButton.remove();
    if (existingSnipeButton) existingSnipeButton.remove();

    const poolLink = card.querySelector('a[href*="/terminal"]');
    if (!poolLink) return;
    const poolUrl = new URL(poolLink.href);
    const tokenMint = poolUrl.searchParams.get("address");
    if (!tokenMint) return;

    let actionArea = card.querySelector("button.ant-btn-text");
    if (migrationContainer) {
      actionArea = migrationContainer;
    }
    if (!actionArea) return;

    if (migrationContainer) {
      addNeoVisionSnipingButton(actionArea, tokenMint);
    } else {
      addNeoVisionBuyButton(actionArea, tokenMint, quickBuyAmount);
    }
  } catch (error) {
    console.error("Failed to add Bloom button:", error);
  }
}

function addNeoVisionSnipingButton(actionArea, tokenMint) {
  const buttonText = "🌸 Snipe";

  const bloomButton = document.createElement("button");
  bloomButton.innerHTML = buttonText;
  bloomButton.type = "button";
  bloomButton.classList.add(snipeButtonClassName, "ant-btn", "ant-btn-text");
  bloomButton.style.border = "1px solid #EEA7ED";
  bloomButton.style.margin = "0px 6px";
  bloomButton.style.zIndex = "1000";
  bloomButton.style.backgroundColor = "#0d0d10";

  bloomButton.onclick = async function (event) {
    event.preventDefault();
    event.stopPropagation();

    const token =
      (await chrome.storage.local.get("bloom.token"))?.["bloom.token"] || "";
    if (!token) {
      return chrome.runtime.sendMessage({
        message: "openTab",
        url: `https://t.me/BloomSolana_bot?start=ref_QT_sniper_${tokenMint}`,
      });
    }

    const order = await createOrder(tokenMint, true, 0, token, "buy");
    if (order?.status === "success") {
      showToast("Order placed successfully!");
    } else {
      showToast("Failed placing order!");
    }
  };

  insertBefore(actionArea, bloomButton);
}

function addNeoVisionBuyButton(actionArea, tokenMint, quickBuyAmount) {
  const buttonText = `🌸 ◎ ${quickBuyAmount}`;

  const bloomButton = document.createElement("button");
  bloomButton.innerHTML = buttonText;
  bloomButton.type = "button";
  bloomButton.classList.add(buyButtonClassName, "ant-btn", "ant-btn-text");
  bloomButton.style.border = "1px solid #EEA7ED";
  bloomButton.style.margin = "0px 6px";
  bloomButton.style.zIndex = "1000";
  bloomButton.style.backgroundColor = "#0d0d10";

  bloomButton.onclick = async function (event) {
    event.preventDefault();
    event.stopPropagation();

    const token =
      (await chrome.storage.local.get("bloom.token"))?.["bloom.token"] || "";
    if (!token) {
      return chrome.runtime.sendMessage({
        message: "openTab",
        url: `https://t.me/BloomSolana_bot?start=ref_QT_ca_${tokenMint}`,
      });
    }

    const order = await createOrder(
      tokenMint,
      false,
      quickBuyAmount,
      token,
      "buy",
    );

    if (order?.status === "success") {
      showToast("Order placed successfully!");
    } else {
      showToast("Failed placing order!");
    }
  };

  insertBefore(actionArea, bloomButton);
}

function addDiscoverTrendingBuyButton(card, quickBuyAmount) {
  try {
    const existingBuyButton = card.querySelector(`.${buyButtonClassName}`);
    if (existingBuyButton) {
      existingBuyButton.remove();
    }

    let actionArea = Array.from(
      card.querySelectorAll("button.ant-btn-text"),
    ).find((b) => b.classList.contains("!flex"));
    let currentNode = card;
    while (!actionArea && currentNode) {
      const sibling = currentNode.nextElementSibling;
      if (sibling && sibling.classList.contains("h-full")) {
        actionArea = sibling.querySelector("button.ant-btn-text");
      }
      currentNode = sibling;
    }

    if (!actionArea) return;
    const buttonClass = buyButtonClassName;
    const buttonText = `🌸 ◎ ${quickBuyAmount}`;

    const bloomButton = document.createElement("button");

    bloomButton.innerHTML = buttonText;
    bloomButton.type = "button";
    bloomButton.classList.add(buttonClass, "ant-btn", "ant-btn-text");

    bloomButton.style.border = "1px solid #EEA7ED";
    bloomButton.style.margin = "0px 6px";
    bloomButton.style.zIndex = "1000";

    const poolLink = card.href;
    if (!poolLink) return;
    const poolUrl = new URL(poolLink);
    const tokenMint = poolUrl.searchParams.get("address");
    if (!tokenMint) return;

    bloomButton.onclick = async function (event) {
      event.preventDefault();
      event.stopPropagation();

      const token =
        (await chrome.storage.local.get("bloom.token"))?.["bloom.token"] || "";
      if (!token) {
        return chrome.runtime.sendMessage({
          message: "openTab",
          url: `https://t.me/BloomSolana_bot?start=ref_QT_ca_${tokenMint}`,
        });
      }

      const order = await createOrder(
        tokenMint,
        false,
        quickBuyAmount,
        token,
        "buy",
      );

      if (order?.status === "success") {
        showToast("Order placed successfully!");
      } else {
        showToast("Failed placing order!");
      }
    };

    insertBefore(actionArea, bloomButton);
  } catch (error) {
    console.error("Failed to add Bloom button:", error);
  }
}

function addTopBarButton() {
  const existingBloomButton = document.querySelector(`.${topBarClassName}`);
  if (existingBloomButton) return;

  const topBar = document.querySelector("div.w-full.no-scrollbar.border-y");
  const lastDiv = topBar?.lastElementChild;
  if (!lastDiv) return;

  const buttonDiv = document.createElement("div");
  buttonDiv.classList.add(
    "flex",
    "items-center",
    "justify-center",
    "text-xs",
    "rounded",
    "p-1",
  );

  const bloomButton = document.createElement("button");
  bloomButton.innerHTML = "🌸 Bloom";
  bloomButton.type = "button";
  bloomButton.classList.add(topBarClassName, "ant-btn", "ant-btn-text");
  bloomButton.style.border = "1px solid #EEA7ED";

  buttonDiv.appendChild(bloomButton);

  bloomButton.onclick = function () {
    const url = new URL(window.location.href);
    const tokenMint = url.searchParams.get("address");
    if (!tokenMint) return showToast("Token not found");

    chrome.runtime.sendMessage({
      message: "openTab",
      url: `https://t.me/BloomSolana_bot?start=ref_QT_ca_${tokenMint}`,
    });
  };

  insertAfter(lastDiv, buttonDiv);
}

function addOneClickButtons(container) {
  const existingOneClickButtons = Array.from(container.querySelectorAll(`.${oneClickButtonClassName}`));
  existingOneClickButtons.forEach((button) => {
    button.remove();
  });

  const style = document.createElement("style");
  style.textContent = `
    .${oneClickButtonClassName} {
      border: 1px solid #EEA7ED;
      color: #EEA7ED;
      z-index: 1000;
      &:hover {
        background-color: #EEA7ED;
        color: hsl(var(--twc-grey-900) / var(--twc-grey-900-opacity, var(--tw-text-opacity)))
      }
    }
  `;
  container.appendChild(style);

  let bloomBuyDiv = null;
  let bloomSellDiv = null;

  const oneClickButtons = Array.from(container.querySelectorAll("button"));
  oneClickButtons.forEach((button) => {
    const clonedButton = button.cloneNode(true);
    clonedButton.classList.add(oneClickButtonClassName);
    const isBuy = !button.textContent.includes('%');
    if (bloomBuyDiv && isBuy) {
      bloomBuyDiv.appendChild(clonedButton);
    } else if (bloomSellDiv && !isBuy) {
      bloomSellDiv.appendChild(clonedButton);
    } else {
      const parent = button.parentElement;
      if (!parent) return;
      if (isBuy) {
        bloomBuyDiv = parent.cloneNode(false);
        insertAfter(parent, bloomBuyDiv);
        bloomBuyDiv.appendChild(clonedButton);
      } else {
        bloomSellDiv = parent.cloneNode(false);
        insertAfter(parent, bloomSellDiv);
        bloomSellDiv.appendChild(clonedButton);
      }
    }

    clonedButton.onclick = async function (event) {
      event.preventDefault();
      event.stopPropagation();

      const url = new URL(window.location.href);
      const tokenMint = url.searchParams.get("address");
      if (!tokenMint) return showToast("Token not found");

      const token =
        (await chrome.storage.local.get("bloom.token"))?.["bloom.token"] || "";
      if (!token) {
        return chrome.runtime.sendMessage({
          message: "openTab",
          url: `https://t.me/BloomSolana_bot?start=ref_QT_ca_${tokenMint}`,
        });
      }

      const isBuy = !button.textContent.includes('%');
      const amount = button.textContent.replace('%', '');
      const order = await createOrder(tokenMint, false, amount, token, isBuy ? "buy" : "sell");
      if (order?.status === "success") {
        showToast("Order placed successfully!");
      } else {
        showToast("Failed placing order!");
      }
    }
  });
}

function addWalletManagerButton(row, reload = false) {
  const existingWalletManagerButton = row.querySelector(`.${walletManagerButtonClassName}`);
  if (existingWalletManagerButton && !reload) return;
  else if (existingWalletManagerButton) existingWalletManagerButton.remove();

  const button = row.querySelector('button.ant-btn-text');
  if (!button) return;

  const clonedButton = button.cloneNode(true);
  clonedButton.classList.add(walletManagerButtonClassName);
  const svg = clonedButton.querySelector('svg');
  const text = document.createElement('p');
  text.textContent = '🌸';
  const span = clonedButton.querySelector('span');
  span.textContent = `◎ ${quickBuyAmount}`;
  insertBefore(svg, text);
  svg.remove();
  clonedButton.style.margin = '0px 4px';
  clonedButton.onclick = async function (event) {
    event.preventDefault();
    event.stopPropagation();

    const poolLink = row.querySelector('a[href*="/terminal"]');
    if (!poolLink) return;
    const poolUrl = new URL(poolLink.href);
    const tokenMint = poolUrl.searchParams.get("address");
    if (!tokenMint) return;

    const token =
      (await chrome.storage.local.get("bloom.token"))?.["bloom.token"] || "";
    if (!token) {
      return chrome.runtime.sendMessage({
        message: "openTab",
        url: `https://t.me/BloomSolana_bot?start=ref_QT_ca_${tokenMint}`,
      });
    }

    const order = await createOrder(
      tokenMint,
      false,
      quickBuyAmount,
      token,
      "buy",
    );

    if (order?.status === "success") {
      showToast("Order placed successfully!");
    } else {
      showToast("Failed placing order!");
    }
  };

  insertBefore(button, clonedButton);
}