function insertAfter(referenceNode, newNode) {
  referenceNode.parentNode.insertBefore(newNode, referenceNode.nextSibling);
}

function insertBefore(referenceNode, newNode) {
  referenceNode.parentNode.insertBefore(newNode, referenceNode);
}

chrome.runtime.onMessage.addListener(async function (request) {
  if (request.message === "bullx-pump-vision") {
    const container = await findPumpVisionContainer();
    if (container) {
      addPumpVisionQTButton();
      const observer = new MutationObserver(() => addPumpVisionQTButton());
      observer.observe(container, { childList: true, subtree: true });
    }
  }
  if (request.message === "bullx-token") {
    const topBar = await findTopBar();
    const buySellContainer = await findBuySellContainer();
    if (topBar || buySellContainer) {
      addCustomBuyButton();
    }
    let currentMigrating = document.querySelector("div.buy-sell-migrating");
    if (buySellContainer) {
      const observer = new MutationObserver(() => {
        const migrating = document.querySelector("div.buy-sell-migrating");
        if (Boolean(migrating) !== Boolean(currentMigrating)) {
          currentMigrating = migrating;
          addCustomBuyButton();
        }
      });
      observer.observe(buySellContainer, { childList: true, subtree: true });
    }
  }
  if (request.message === "bullx-home") {
    const main = await findMain();
    if (main) {
      await handleBullxHome();
    }
    const observer = new MutationObserver(() => handleBullxHome());
    observer.observe(main, { childList: true });
  }
});

async function findMain(timeout = 5000) {
  for (let i = 0; i < timeout / 500; i++) {
    const main = document.querySelector(".flex.overflow-hidden.no-scrollbar");
    if (main) return main;
    await new Promise((r) => setTimeout(r, 500));
  }
  return null;
}

async function findPumpVisionContainer(timeout = 5000) {
  for (let i = 0; i < timeout / 500; i++) {
    const container = document.querySelector("div.grid");
    if (container) return container;
    await new Promise((r) => setTimeout(r, 500));
  }
  return null;
}

async function findTableContainer(timeout = 5000) {
  for (let i = 0; i < timeout / 500; i++) {
    const container = document.querySelector(".ant-table-tbody");
    if (container) return container;
    await new Promise((r) => setTimeout(r, 500));
  }
  return null;
}

async function findTopBar(timeout = 5000) {
  for (let i = 0; i < timeout / 500; i++) {
    const topBar = document.querySelector("div.grid.no-scrollbar");
    if (topBar) return topBar;
    await new Promise((r) => setTimeout(r, 500));
  }
  return null;
}

async function findBuySellContainer(timeout = 5000) {
  for (let i = 0; i < timeout / 500; i++) {
    const container = document.querySelector("div.ant-drawer-content-wrapper");
    if (container) return container;
    await new Promise((r) => setTimeout(r, 500));
  }
  return null;
}

function addCustomBuyButton() {
  try {
    const migrationContainer = document.querySelector("div.buy-sell-migrating");
    const previousSnipingButton = document.querySelector(".bloom-snipe-qt-btn");
    const previousBuyButton = document.querySelector(".bloom-buy-qt-btn");

    if (previousSnipingButton) {
      previousSnipingButton.remove();
    }
    if (previousBuyButton) {
      previousBuyButton.remove();
    }

    if (migrationContainer) {
      const migrationText = Array.from(
        migrationContainer.querySelectorAll("p"),
      ).filter((p) => p.textContent.includes("may take a few minutes"))?.[0];
      if (!migrationText) return;

      const bloomButton = document.createElement("button");
      bloomButton.innerHTML = "🌸 Snipe with Bloom";
      bloomButton.type = "button";
      bloomButton.classList.add(
        "bloom-snipe-qt-btn",
        "ant-btn",
        "ant-btn-text",
      );
      bloomButton.style.marginTop = "6px";
      bloomButton.style.marginBottom = "-24px";
      bloomButton.style.border = "1px solid #EEA7ED";

      bloomButton.onclick = function () {
        const url = new URL(window.location.href);
        const tokenMint = url.searchParams.get("address");

        chrome.runtime.sendMessage({
          message: "openTab",
          url: `https://t.me/BloomSolana_bot?start=ref_QT_sniper_${tokenMint}`,
        });
      };

      insertBefore(migrationText, bloomButton);
    }

    const topBar = document.querySelector("div.grid.no-scrollbar");
    const lastDiv = topBar?.lastElementChild;
    if (!lastDiv) return;

    const bloomButton = document.createElement("button");
    bloomButton.innerHTML = "🌸 Bloom";
    bloomButton.type = "button";
    bloomButton.classList.add("bloom-buy-qt-btn", "ant-btn", "ant-btn-text");
    bloomButton.style.marginTop = "6px";
    bloomButton.style.marginBottom = "-24px";
    bloomButton.style.border = "1px solid #EEA7ED";

    bloomButton.onclick = function () {
      const url = new URL(window.location.href);
      const tokenMint = url.searchParams.get("address");

      chrome.runtime.sendMessage({
        message: "openTab",
        url: `https://t.me/BloomSolana_bot?start=ref_QT_ca_${tokenMint}`,
      });
    };

    insertAfter(lastDiv, bloomButton);
  } catch (error) {
    console.error("Failed to add Custom Buy button:", error);
  }
}

function addPumpVisionQTButton() {
  try {
    const cards = Array.from(document.querySelectorAll("div.pump-card"));

    cards.forEach((card) => {
      const isMigrating = Array.from(card.querySelectorAll("span")).some(
        (span) => span.textContent === "Migrating...",
      );

      const existingBuyButton = card.querySelector(".bloom-buy-qt-btn");
      const existingSnipeButton = card.querySelector(".bloom-snipe-qt-btn");

      if (existingBuyButton || existingSnipeButton) {
        if (isMigrating && existingBuyButton) {
          existingBuyButton.remove();
        } else if (!isMigrating && existingSnipeButton) {
          existingSnipeButton.remove();
        } else {
          return;
        }
      }

      const poolLink = card.querySelector('a[href*="/terminal"]');
      if (!poolLink) return;

      let actionArea = card.querySelector("button");
      if (isMigrating) {
        actionArea = Array.from(card.querySelectorAll("span")).find(
          (span) => span.textContent === "Migrating...",
        );
      }
      if (!actionArea) return;

      const buttonClass = isMigrating
        ? "bloom-snipe-qt-btn"
        : "bloom-buy-qt-btn";
      const buttonText = isMigrating ? "🌸 Snipe" : "🌸 Buy";

      const bloomButton = document.createElement("button");

      bloomButton.innerHTML = buttonText;
      bloomButton.type = "button";
      bloomButton.classList.add(buttonClass, "ant-btn", "ant-btn-text");

      bloomButton.style.border = "1px solid #EEA7ED";
      bloomButton.style.margin = "0px 6px";
      bloomButton.style.zIndex = "1000";

      const poolUrl = new URL(poolLink.href);
      const tokenMint = poolUrl.searchParams.get("address");

      bloomButton.onclick = function (event) {
        event.preventDefault();
        event.stopPropagation();
        chrome.runtime.sendMessage({
          message: "openTab",
          url: `https://t.me/BloomSolana_bot?start=ref_QT_${
            isMigrating ? "sniper" : "ca"
          }_${tokenMint}`,
        });
      };

      insertBefore(actionArea, bloomButton);
    });
  } catch (error) {
    console.log("Failed to add Bloom button:", error);
  }
}

function addDiscoverTrendingQTButton() {
  try {
    const cards = Array.from(document.querySelectorAll("tr.ant-table-row"));

    cards.forEach((card) => {
      const existingBuyButton = card.querySelector(".bloom-buy-qt-btn");
      if (existingBuyButton) {
        return;
      }

      const allTds = card.querySelectorAll("td");
      const lastTd = allTds[allTds.length - 1];
      const actionArea = lastTd?.querySelector("button");

      if (!actionArea) return;

      const buttonClass = "bloom-buy-qt-btn";
      const buttonText = "🌸 Buy";

      const bloomButton = document.createElement("button");

      bloomButton.innerHTML = buttonText;
      bloomButton.type = "button";
      bloomButton.classList.add(buttonClass, "ant-btn", "ant-btn-text");

      bloomButton.style.border = "1px solid #EEA7ED";
      bloomButton.style.margin = "0px 6px";
      bloomButton.style.zIndex = "1000";

      const [poolId, chainId] = card.getAttribute("data-row-key")?.split("_");
      if (!poolId || chainId !== "1399811149") return;

      bloomButton.onclick = function (event) {
        event.preventDefault();
        event.stopPropagation();
        chrome.runtime.sendMessage({
          message: "openTab",
          url: "https://t.me/BloomSolana_bot?start=ref_QT_ca_" + poolId,
        });
      };

      insertBefore(actionArea, bloomButton);
    });
  } catch (error) {
    console.log("Failed to add Bloom button:", error);
  }
}

async function handleBullxHome() {
  const container = await findTableContainer();
  if (container) {
    addDiscoverTrendingQTButton();
    const observer = new MutationObserver(() => addDiscoverTrendingQTButton());
    observer.observe(container, { childList: true, subtree: true });
  }

  return container ?? null;
}
