function insertAfter(referenceNode, newNode) {
  referenceNode.parentNode.insertBefore(newNode, referenceNode.nextSibling);
}

function insertBefore(referenceNode, newNode) {
  referenceNode.parentNode.insertBefore(newNode, referenceNode);
}

chrome.runtime.onMessage.addListener(async function (request) {
  if (request.message === "dexscreener-token") {
    const container = await findTokenContainer();
    if (container) {
      addTokenButtons(container.container, container.hasMetadata);
    }
    const main = document.querySelector("main");
    const observer = new MutationObserver(async (o) => {
      for (const m of o) {
        if (
          m.removedNodes.length > 0 &&
          Array.from(m.removedNodes).find(
            (n) =>
              n.nodeName.toLowerCase() === "div" &&
              !n.classList?.contains("bloom-buy-qt-btn-div"),
          ) &&
          !m.target?.classList?.contains("chakra-stack")
        ) {
          const container = await findTokenContainer();
          if (container) {
            addTokenButtons(container.container, container.hasMetadata);
          }
        }
      }
    });
    observer.observe(main, { childList: true, subtree: true });
  }
});

async function findTokenContainer(timeout = 10000) {
  for (let i = 0; i < timeout / 500; i++) {
    const fullContainer = [
      document.querySelector('div[style="overflow-anchor: none;"]'),
      document.querySelector('div[style="overflow-anchor:none"]'),
    ].find(Boolean);
    if (fullContainer) {
      const realChildren = Array.from(fullContainer.children).filter((c) =>
        Array.from(c.classList).every((c) => !c.includes("bloom")),
      );
      const hasChart = realChildren.some((c) =>
        c.innerHTML.includes("Embed this chart"),
      );
      const hasMetadataLength = hasChart ? 8 : 7;
      const hasMetadata = realChildren.length === hasMetadataLength;
      const statsDiv = hasMetadata ? realChildren[2] : realChildren[1];
      if (statsDiv) {
        return { container: statsDiv, hasMetadata };
      }
    }
    await new Promise((r) => setTimeout(r, 500));
  }
  return null;
}

function addTokenButtons(container, hasMetadata) {
  try {
    const existingButton = document.querySelector(".bloom-buy-qt-btn");
    const existingDiv = document.querySelector(".bloom-buy-qt-btn-div");
    if (existingButton) existingButton.remove();
    if (existingDiv) existingDiv.remove();

    const buttonDiv = document.createElement("div");
    buttonDiv.classList.add("bloom-buy-qt-btn-div");
    buttonDiv.style.display = "flex";
    buttonDiv.style.justifyContent = "center";
    buttonDiv.style.alignItems = "center";
    buttonDiv.style.width = "100%";
    buttonDiv.style.marginTop = hasMetadata ? "24px" : "12px";

    const bloomButton = document.createElement("button");
    bloomButton.style.padding = "2px";
    bloomButton.style.borderRadius = "6px";
    bloomButton.style.margin = "auto";
    bloomButton.style.width = "50%";
    bloomButton.style.border = "1px solid #EEA7ED";
    bloomButton.classList.add("bloom-buy-qt-btn");
    bloomButton.textContent = "🌸 Bloom";

    buttonDiv.appendChild(bloomButton);

    const poolAddress = [...document.getElementsByClassName("chakra-text")]
      ?.find((e) => e.innerText === "Pair")
      ?.getAttribute("title");
    if (!poolAddress) return;

    bloomButton.onclick = function (event) {
      event.preventDefault();
      event.stopPropagation();
      chrome.runtime.sendMessage({
        message: "openTab",
        url: `https://t.me/BloomSolana_bot?start=ref_QT_ca_${poolAddress}`,
      });
    };

    insertBefore(container, buttonDiv);
  } catch (error) {
    console.error("Failed to add Dexscreener token buttons:", error);
  }
}
