function insertAfter(referenceNode, newNode) {
  referenceNode.parentNode.insertBefore(newNode, referenceNode.nextSibling);
}

function insertBefore(referenceNode, newNode) {
  referenceNode.parentNode.insertBefore(newNode, referenceNode);
}

chrome.runtime.onMessage.addListener(async function (request) {
  if (request.event === "onUpdated")
    await new Promise((r) => setTimeout(r, 1000));
  const tables = await findGmgnTables();
  if (!tables || tables.length === 0) return;
  if (request.message === "gmgn-empty-tab") {
    request.message = tables.length > 1 ? "gmgn-meme" : "gmgn-home";
  }
  if (request.message === "gmgn-home") {
    const container = tables[0];
    if (container) {
      addTrendingButtons();
      const observer = new MutationObserver(() => addTrendingButtons());
      observer.observe(container, { childList: true });
    }
  }
  if (request.message === "gmgn-meme") {
    if (tables && tables.length > 0) {
      const allRows = Array.from(document.querySelectorAll(".g-table-row"));
      allRows.forEach((r) => addMemeButtons(r));
      tables.forEach((table) => {
        const observer = new MutationObserver((m) => {
          const addedRows = Array.from(m)
            .map((m) =>
              Array.from(m.addedNodes).concat(Array.from(m.removedNodes)),
            )
            .flat()
            .filter((r) => r.classList?.contains("g-table-row"));
          if (addedRows.length === 0) {
            const allCurrentRows = Array.from(
              document.querySelectorAll(".g-table-row"),
            );
            allCurrentRows.forEach((r) => addMemeButtons(r));
          } else {
            addedRows.forEach((r) => addMemeButtons(r));
          }
        });
        observer.observe(table, { childList: true, subtree: true });
      });
    }
  }
  if (request.message === "gmgn-token") {
    const container = await findTokenContainer();
    if (container) {
      addTokenButtons(container, request.url);
    }
  }
});

async function findGmgnTables(timeout = 5000) {
  for (let i = 0; i < timeout / 500; i++) {
    const tables = Array.from(
      document.querySelectorAll(".g-table-tbody-virtual-holder-inner"),
    );
    if (tables.length > 0) {
      return tables;
    }
    await new Promise((r) => setTimeout(r, 500));
  }
  return null;
}

async function findTokenContainer(timeout = 5000) {
  for (let i = 0; i < timeout / 500; i++) {
    const center = document.querySelector("#tokenCenter");
    if (center) return center.nextElementSibling;
    await new Promise((r) => setTimeout(r, 500));
  }
  return null;
}

function addTrendingButtons() {
  try {
    const rows = Array.from(document.querySelectorAll(".g-table-row"));
    rows.forEach((row) => {
      const existingBuyButton = row.querySelector(".bloom-buy-qt-btn");

      if (existingBuyButton) {
        return;
      }

      const rowDataKey = row.getAttribute("data-row-key");
      const tokenMint = rowDataKey.includes("_")
        ? rowDataKey.split("_")[1]
        : rowDataKey;
      if (!tokenMint) return;

      const actionArea = row.querySelector(".g-table-cell-fix-right-first");
      if (!actionArea) return;
      const lastChild = actionArea.lastChild;
      if (!lastChild) return;
      const classDiv = Array.from(
        row.querySelectorAll('div[datatype="out"]'),
      ).find((div) => !div.querySelector("a[href]"));
      if (!classDiv) return;

      const currentStyle = actionArea.getAttribute("style");
      actionArea.setAttribute(
        "style",
        currentStyle
          ? `${currentStyle}; flex-direction: column; width: 100%;`
          : "flex-direction: column; width: 100%;",
      );

      const bloomButton = document.createElement("button");
      bloomButton.classList.add("bloom-buy-qt-btn", classDiv.classList[0]);
      bloomButton.textContent = "🌸 Buy";

      bloomButton.onclick = function (event) {
        event.preventDefault();
        event.stopPropagation();
        chrome.runtime.sendMessage({
          message: "openTab",
          url: `https://t.me/BloomSolana_bot?start=ref_QT_ca_${tokenMint}`,
        });
      };

      insertAfter(lastChild, bloomButton);
    });
  } catch (error) {
    console.error("Failed to add GMGN table buttons:", error);
  }
}

function addMemeButtons(card) {
  try {
    const migrationText = Array.from(card.querySelectorAll("div")).find(
      (div) => {
        return Array.from(div.childNodes)
          .filter((node) => node.nodeType === Node.TEXT_NODE)
          .some((node) => node.textContent.trim() === "Seeding");
      },
    );
    const isMigrating = !!migrationText;
    const existingBuyButton = card.querySelector(".bloom-buy-qt-btn");
    const existingSnipeButton = card.querySelector(".bloom-snipe-qt-btn");

    if (existingBuyButton || existingSnipeButton) {
      if (isMigrating && existingBuyButton) {
        existingBuyButton.remove();
      } else if (!isMigrating && existingSnipeButton) {
        existingSnipeButton.remove();
      } else {
        return;
      }
    }

    const rowDataKey = card.getAttribute("data-row-key");
    const tokenMint = rowDataKey.includes("_")
      ? rowDataKey.split("_")[1]
      : rowDataKey;
    if (!tokenMint) return;

    let actionArea = card.querySelector('div[datatype="out"]');
    let classDiv = actionArea;
    if (!actionArea) {
      actionArea = migrationText;
      classDiv = Array.from(
        document.querySelectorAll('div[datatype="out"]'),
      ).find((d) =>
        Array.from(d.childNodes).some(
          (c) => c.nodeName.toLowerCase() === "svg",
        ),
      );
    }
    if (!actionArea) return;

    const bloomButton = document.createElement("button");
    bloomButton.classList.add(
      isMigrating ? "bloom-snipe-qt-btn" : "bloom-buy-qt-btn",
      classDiv?.classList?.[0],
    );
    bloomButton.textContent = isMigrating ? "🌸 Snipe" : "🌸 Buy";
    if (isMigrating) {
      bloomButton.setAttribute("style", "margin-right: 6px;");
    }

    bloomButton.onclick = function (event) {
      event.preventDefault();
      event.stopPropagation();
      chrome.runtime.sendMessage({
        message: "openTab",
        url: `https://t.me/BloomSolana_bot?start=ref_QT_${
          isMigrating ? "sniper" : "ca"
        }_${tokenMint}`,
      });
    };

    insertBefore(actionArea, bloomButton);
  } catch (error) {
    console.error("Failed to add GMGN meme buttons:", error);
  }
}

function addTokenButtons(container, url) {
  try {
    const existingButton = container.querySelector(".bloom-buy-qt-btn");
    if (existingButton) existingButton.remove();

    const firstChild = container.firstChild;
    if (!firstChild) return;

    const bloomButton = document.createElement("button");
    bloomButton.classList.add("bloom-buy-qt-btn");
    bloomButton.style.border = "1px solid #EEA7ED";
    bloomButton.style.padding = "2px";
    bloomButton.style.borderRadius = "6px";
    bloomButton.style.margin = "auto";
    bloomButton.style.width = "50%";
    bloomButton.textContent = "🌸 Bloom";

    const tokenMint = url.split("token/")[1]?.split("?")[0];
    if (!tokenMint) return;

    bloomButton.onclick = function (event) {
      event.preventDefault();
      event.stopPropagation();
      chrome.runtime.sendMessage({
        message: "openTab",
        url: `https://t.me/BloomSolana_bot?start=ref_QT_ca_${tokenMint}`,
      });
    };

    insertAfter(firstChild, bloomButton);
  } catch (error) {
    console.error("Failed to add GMGN token buttons:", error);
  }
}
