const activeObservers = {};

// LISTENER

let currentPage, quickBuyAmount;
chrome.runtime.onMessage.addListener(async function (request) {
  if (request.message === "photon-memescope") {
    const token =
      (await chrome.storage.local.get("bloom.token"))?.["bloom.token"] || "";
    if (!token && request.event === "onCompleted") {
      showToast(
        "Log in to the Bloom extension to enhance your experience!",
        "top-center",
      );
    }
    const observers = activeObservers["memescope"] || [];
    observers.forEach((observer) => observer.disconnect());
    currentPage = request.message;
    handleMemescope();
  }
  if (request.message === "photon-token") {
    const token =
      (await chrome.storage.local.get("bloom.token"))?.["bloom.token"] || "";
    if (!token && request.event === "onCompleted") {
      showToast(
        "Log in to the Bloom extension to enhance your experience!",
        "top-center",
      );
    }
    const observers = activeObservers["token"] || [];
    observers.forEach((observer) => observer.disconnect());
    currentPage = request.message;
    handleToken();
  }
  if (
    request.message === "photon-discover" ||
    request.message === "photon-trending"
  ) {
    const token =
      (await chrome.storage.local.get("bloom.token"))?.["bloom.token"] || "";
    if (!token && request.event === "onCompleted") {
      showToast(
        "Log in to the Bloom extension to enhance your experience!",
        "top-center",
      );
    }
    const observers = activeObservers["discover-trending"] || [];
    observers.forEach((observer) => observer.disconnect());
    currentPage = request.message;
    handleDiscoverTrending();
  }
  if (request.message === "quickBuyAmount") {
    if (
      currentPage === "photon-trending" ||
      currentPage === "photon-discover"
    ) {
      const container = await findTokenContainer();
      const cards = Array.from(container.querySelectorAll("a[href*='/lp/']"));
      quickBuyAmount =
        (await chrome.storage.local.get("bloom.quickBuyAmount"))?.[
          "bloom.quickBuyAmount"
        ] || 0;
      cards.forEach((card) =>
        addDiscoverTrendingBuyButton(card, quickBuyAmount),
      );
    } else if (currentPage === "photon-memescope") {
      const cards = Array.from(document.querySelectorAll("a[href*='/lp/']"));
      quickBuyAmount =
        (await chrome.storage.local.get("bloom.quickBuyAmount"))?.[
          "bloom.quickBuyAmount"
        ] || 0;
      cards.forEach((card) =>
        addMemescopeButtons(card.parentElement, quickBuyAmount, true),
      );
    }
  }
});

// UTILS

function insertAfter(referenceNode, newNode) {
  referenceNode.parentNode.insertBefore(newNode, referenceNode.nextSibling);
}

function insertBefore(referenceNode, newNode) {
  referenceNode.parentNode.insertBefore(newNode, referenceNode);
}

// ORDERS

async function createOrder(address, isSnipe, amount, authToken, side) {
  try {
    const res = await fetch(
      `https://extension.bloombot.app/${isSnipe ? "snipe" : "swap"}`,
      {
        method: "POST",
        headers: {
          "Content-Type": "application/json",
        },
        body: JSON.stringify(
          isSnipe
            ? {
                addr: address,
                auth: authToken,
              }
            : {
                addr: address,
                isPool: true,
                amt: parseFloat(amount).toString(),
                auth: authToken,
                side,
              },
        ),
      },
    );

    return res.json();
  } catch (error) {
    console.error("Failed to create order:", error);
    return null;
  }
}

// TOAST

function showToast(message, position = "bottom-right") {
  const previousToasts = document.querySelectorAll(".bloom-toast");
  previousToasts.forEach((toast) => toast.remove());

  const toast = document.createElement("div");
  toast.style.position = "fixed";
  if (position === "bottom-right") {
    toast.style.bottom = "20px";
    toast.style.right = "20px";
  } else if (position === "top-center") {
    toast.style.top = "20px";
    toast.style.left = "50%";
    toast.style.transform = "translateX(-50%)";
  }
  toast.style.backgroundColor = "#181921";
  toast.style.color = "#EEA7ED";
  toast.style.padding = "12px 24px";
  toast.style.borderRadius = "5px";
  toast.style.zIndex = "10000";
  toast.style.fontSize = "14px";
  toast.style.fontWeight = "600";
  toast.style.border = "1px solid #EEA7ED";
  toast.textContent = "🌸 " + message;
  toast.classList.add("bloom-toast");

  document.body.appendChild(toast);

  setTimeout(() => {
    toast.style.transition = "opacity 0.5s ease-out";
    toast.style.opacity = "0";
    setTimeout(() => toast.remove(), 500);
  }, 3000);
}

// FINDERS

async function findTokenContainer(isMemescope = false, timeout = 5000) {
  for (let i = 0; i < timeout / 500; i++) {
    const lpLinks = Array.from(document.querySelectorAll('a[href*="/lp/"]'));
    if (lpLinks.length > 0) {
      if (isMemescope) {
        const containers = Array.from(
          new Set(lpLinks.map((n) => n.parentElement.parentElement)),
        ).filter((n) => !n.classList.contains("js-watching-list"));
        if (containers.length) return containers;
      } else {
        const container = lpLinks[0].parentElement;
        if (container) {
          return container;
        }
      }
    }
    await new Promise((r) => setTimeout(r, 500));
  }
  return null;
}

async function findTopBar(timeout = 5000) {
  for (let i = 0; i < timeout / 500; i++) {
    const topBar = document.querySelector(".p-show__bar__row");
    if (topBar) {
      const lastDiv = topBar.querySelector(".l-col-md-auto:last-of-type");
      if (lastDiv) return lastDiv;
    }
    await new Promise((r) => setTimeout(r, 500));
  }
  return null;
}

async function findBuySellContainer(timeout = 5000) {
  for (let i = 0; i < timeout / 500; i++) {
    const container = document.querySelector("div.js-show__trade-tabs");
    if (container) return container;
    await new Promise((r) => setTimeout(r, 500));
  }
  return null;
}

// HANDLERS

async function handleDiscoverTrending() {
  const container = await findTokenContainer();
  quickBuyAmount =
    (await chrome.storage.local.get("bloom.quickBuyAmount"))?.[
      "bloom.quickBuyAmount"
    ] || 0;
  if (container) {
    const cards = Array.from(container.querySelectorAll("a[href*='/lp/']"));
    cards.forEach((card) => addDiscoverTrendingBuyButton(card, quickBuyAmount));
    const observer = new MutationObserver((mutations) => {
      mutations.forEach((mutation) => {
        mutation.addedNodes.forEach((node) => {
          if (
            node.nodeName &&
            node.nodeName.toLowerCase() === "a" &&
            node.href.includes("/lp/")
          ) {
            addDiscoverTrendingBuyButton(node, quickBuyAmount);
          }
        });
      });
    });

    observer.observe(container, { childList: true });
    activeObservers["discover-trending"]
      ? activeObservers["discover-trending"].push(observer)
      : (activeObservers["discover-trending"] = [observer]);

    const containerParent = container.parentElement;
    if (containerParent) {
      const parentObserver = new MutationObserver(() => {
        observer.disconnect();
        parentObserver.disconnect();
        return handleDiscoverTrending();
      });
      parentObserver.observe(containerParent, { childList: true });
      activeObservers["discover-trending"]
        ? activeObservers["discover-trending"].push(parentObserver)
        : (activeObservers["discover-trending"] = [parentObserver]);
    }
  }
}

async function handleMemescopeContainer(container, idx) {
  quickBuyAmount =
    (await chrome.storage.local.get("bloom.quickBuyAmount"))?.[
      "bloom.quickBuyAmount"
    ] || 0;
  const cards = Array.from(container.querySelectorAll("a[href*='/lp/']")).map(
    (n) => n.parentElement,
  );
  cards.forEach((card) => addMemescopeButtons(card, quickBuyAmount));
  const observer = new MutationObserver((mutations) => {
    mutations.forEach((mutation) => {
      mutation.addedNodes.forEach((node) => {
        if (node.nodeName && node.nodeName.toLowerCase() === "div") {
          const lpLink = node.querySelector("a[href*='/lp/']");
          if (lpLink) {
            addMemescopeButtons(node, quickBuyAmount);
          }
        }

        if (node.nodeName && node.nodeName.toLowerCase() === "span") {
          const isMigrating = node.textContent === "Migrating...";
          if (isMigrating) {
            const card = mutation.target?.offsetParent?.offsetParent;
            if (card) {
              addMemescopeButtons(card, quickBuyAmount, true);
            }
          }
        }
      });
    });
  });

  observer.observe(container, { childList: true, subtree: true });
  activeObservers["memescope"]
    ? activeObservers["memescope"].push(observer)
    : (activeObservers["memescope"] = [observer]);

  const containerParent = container.parentElement;
  if (containerParent) {
    const parentObserver = new MutationObserver(() => {
      observer.disconnect();
      parentObserver.disconnect();
      return handleMemescope(idx);
    });

    parentObserver.observe(containerParent, { childList: true });
    activeObservers["memescope"]
      ? activeObservers["memescope"].push(parentObserver)
      : (activeObservers["memescope"] = [parentObserver]);
  }
}

async function handleMemescope(idx) {
  const containers = await findTokenContainer(true);
  if (idx && containers[idx]) {
    handleMemescopeContainer(containers[idx], idx);
  } else {
    containers.forEach((container, idx) =>
      handleMemescopeContainer(container, idx),
    );
  }
}

async function handleToken() {
  try {
    const topBar = await findTopBar();
    const buySellContainer = await findBuySellContainer();
    if (!topBar || !buySellContainer) return;
    let currentMigrating = document.querySelector("div.p-show__migration");
    addTokenButtons(currentMigrating);
    if (buySellContainer) {
      const observer = new MutationObserver((mutations) => {
        if (
          mutations.every(
            (m) =>
              m.target.nodeName && m.target.nodeName.toLowerCase() === "span",
          )
        )
          return;
        const migrating = document.querySelector("div.p-show__migration");
        if (Boolean(migrating) !== Boolean(currentMigrating)) {
          currentMigrating = migrating;
          addTokenButtons(currentMigrating);
        }
      });

      observer.observe(buySellContainer, { childList: true, subtree: true });
      activeObservers["token"]
        ? activeObservers["token"].push(observer)
        : (activeObservers["token"] = [observer]);
    }
  } catch (error) {
    console.error("Failed to add Token buttons:", error);
  }
}

// INJECTORS

function addTokenButtons(migrationContainer) {
  const previousSnipingButton = document.querySelector(".bloom-snipe-qt-btn");
  const previousBuyButton = document.querySelector(".bloom-buy-qt-btn");

  if (previousSnipingButton) {
    previousSnipingButton.remove();
  }
  if (previousBuyButton) {
    previousBuyButton.remove();
  }

  if (migrationContainer) {
    addTokenSnipeButton(migrationContainer);
  }

  addTokenTopBarButton();
  addTokenBuyButton();
  addTokenSellButton();
}

function addTokenSnipeButton(migrationContainer) {
  const previousSnipeButton = document.querySelector(".bloom-snipe-qt-btn");
  if (previousSnipeButton) previousSnipeButton.remove();

  const migrationText = migrationContainer.querySelector("h2");
  if (!migrationText) return;

  const tokenMint = document
    .querySelector(".js-copy-to-clipboard:not(.p-show__bar__copy)")
    ?.getAttribute("data-address");
  if (!tokenMint) return;

  const bloomButton = document.createElement("button");
  bloomButton.innerHTML = "🌸 Snipe with Bloom";
  bloomButton.type = "button";
  bloomButton.classList.add("bloom-snipe-qt-btn", "c-btn", "c-btn--lt");
  bloomButton.style.height = "32px";
  bloomButton.style.padding = "0 10px";
  bloomButton.style.marginBottom = "12px";

  bloomButton.onclick = async function () {
    const token =
      (await chrome.storage.local.get("bloom.token"))?.["bloom.token"] || "";
    if (!token) {
      return chrome.runtime.sendMessage({
        message: "openTab",
        url: `https://t.me/BloomSolana_bot?start=ref_QT_sniper_${tokenMint}`,
      });
    }

    const order = await createOrder(tokenMint, true, 0, token, "buy");
    if (order?.status === "success") {
      showToast("Order placed successfully!");
    } else {
      showToast("Failed placing order!");
    }
  };

  insertBefore(migrationText, bloomButton);
}

function addTokenTopBarButton() {
  const previousTopBarButton = document.querySelector(".bloom-top-qt-btn");
  if (previousTopBarButton) return;

  const topBar = document.querySelector(".p-show__bar__row");
  if (!topBar) return;
  const lastDiv = topBar.querySelector(".l-col-md-auto:last-of-type");
  if (!lastDiv) return;

  const bloomButton = document.createElement("button");
  bloomButton.innerHTML = "🌸 Trade on Bloom";
  bloomButton.type = "button";
  bloomButton.classList.add("bloom-top-qt-btn", "c-btn", "c-btn--lt");
  bloomButton.style.height = "32px";
  bloomButton.style.padding = "0 10px";

  bloomButton.onclick = function () {
    const urlParts = window.location.href.split("/lp/");
    let poolId = "";
    if (urlParts.length > 1) {
      poolId = urlParts[1].split("?")[0];
    }

    chrome.runtime.sendMessage({
      message: "openTab",
      url: "https://t.me/BloomSolana_bot?start=ref_QT_ca_" + poolId,
    });
  };

  insertAfter(lastDiv, bloomButton);
}

function addTokenBuyButton() {
  const previousBuyButton = document.querySelector(".bloom-buy-qt-btn");
  if (previousBuyButton) previousBuyButton.remove();

  const photonBuyButtons = document.querySelectorAll(
    "button.js-buy-btn.js-show__trade-form__submit",
  );
  if (photonBuyButtons.length === 0) return;

  photonBuyButtons.forEach((photonBuyButton) => {
    const bloomButton = photonBuyButton.cloneNode(true);
    const firstDiv = bloomButton.querySelector("div");
    if (!firstDiv) return;
    firstDiv.textContent = "🌸";
    firstDiv.classList.remove("c-icon--flash");
    firstDiv.style.paddingTop = "2px";
    firstDiv.style.backgroundColor = "transparent";
    bloomButton.innerHTML = bloomButton.innerHTML.replace(
      "Quick Buy",
      "Bloom Buy",
    );
    bloomButton.classList.add("bloom-buy-qt-btn", "c-btn", "c-btn--lt");
    bloomButton.onclick = async function (event) {
      event.preventDefault();
      event.stopPropagation();

      const urlParts = window.location.href.split("/lp/");
      let poolId = "";
      if (urlParts.length > 1) {
        poolId = urlParts[1].split("?")[0];
      }

      const token =
        (await chrome.storage.local.get("bloom.token"))?.["bloom.token"] || "";
      if (!token) {
        return chrome.runtime.sendMessage({
          message: "openTab",
          url: `https://t.me/BloomSolana_bot?start=ref_QT_ca_${poolId}`,
        });
      }

      const amount = bloomButton.querySelector(
        ".js-buy-btn__amount",
      )?.textContent;
      if (!amount) {
        return showToast("Please enter a valid amount!");
      }

      const order = await createOrder(poolId, false, amount, token, "buy");

      if (order?.status === "success") {
        showToast("Order placed successfully!");
      } else {
        showToast("Failed placing order!");
      }
    };
    insertAfter(photonBuyButton, bloomButton);
  });
}

function addTokenSellButton() {
  const previousSellButton = document.querySelector(".bloom-sell-qt-btn");
  if (previousSellButton) return;

  const photonSellButtons = document.querySelectorAll(
    "button.js-sell-btn.js-show__trade-form__submit",
  );
  if (photonSellButtons.length === 0) return;

  photonSellButtons.forEach((photonSellButton) => {
    const container = photonSellButton.closest(".js-show__trade-form");
    if (!container) return;

    const pctButtons = container.querySelectorAll(".js-price-form__btn");
    if (pctButtons.length === 0) return;
    const pctInput = container.querySelector(".js-price-form__input");
    if (!pctInput) return;

    const bloomButton = photonSellButton.cloneNode(true);
    bloomButton.textContent = "🌸 Bloom Sell";
    bloomButton.classList.add("bloom-sell-qt-btn", "c-btn", "c-btn--lt");
    bloomButton.onclick = async function (event) {
      event.preventDefault();
      event.stopPropagation();

      const urlParts = window.location.href.split("/lp/");
      let poolId = "";
      if (urlParts.length > 1) {
        poolId = urlParts[1].split("?")[0];
      }

      const token =
        (await chrome.storage.local.get("bloom.token"))?.["bloom.token"] || "";
      if (!token) {
        return chrome.runtime.sendMessage({
          message: "openTab",
          url: `https://t.me/BloomSolana_bot?start=ref_QT_ca_${poolId}`,
        });
      }

      let selectedPct = container
        .querySelector(".js-price-form__btn.is-selected")
        ?.getAttribute("data-value");
      if (!selectedPct) {
        selectedPct = Array.from(
          container.querySelectorAll(
            '.js-price-form__input[data-type="percent"]',
          ),
        ).find((n) => n.value)?.value;
        if (!selectedPct || isNaN(parseFloat(selectedPct))) {
          return showToast("Please select a valid percentage!");
        }
      }

      const order = await createOrder(
        poolId,
        false,
        selectedPct,
        token,
        "sell",
      );
      if (order?.status === "success") {
        showToast("Order placed successfully!");
      } else {
        showToast("Failed placing order!");
      }
    };

    pctButtons.forEach((pctButton) => {
      const observer = new MutationObserver((mutations) => {
        mutations.forEach((mutation) => {
          if (mutation.target?.classList?.contains("is-selected")) {
            const selectedPct = mutation.target?.getAttribute("data-value");
            if (selectedPct) {
              bloomButton.textContent = `🌸 Bloom Sell ${selectedPct}%`;
            }
          }
        });
      });
      observer.observe(pctButton, { attributes: true });
    });

    pctInput.addEventListener("input", () => {
      if (!pctInput.value) {
        bloomButton.textContent = "🌸 Bloom Sell";
      } else {
        bloomButton.textContent = `🌸 Bloom Sell ${pctInput.value}%`;
      }
    });

    insertAfter(photonSellButton, bloomButton);
  });
}

function addMemescopeButtons(card, quickBuyAmount, reload = false) {
  try {
    const isMigrating = Array.from(card.querySelectorAll("span")).some(
      (span) => span.textContent === "Migrating...",
    );
    const existingBuyButton = card.querySelector(".bloom-buy-qt-btn");
    const existingSnipeButton = card.querySelector(".bloom-snipe-qt-btn");

    if (existingBuyButton || existingSnipeButton) {
      if (reload && existingBuyButton) {
        existingBuyButton.remove();
      } else if (reload && existingSnipeButton) {
        existingSnipeButton.remove();
      } else if (isMigrating && existingBuyButton) {
        existingBuyButton.remove();
      } else if (!isMigrating && existingSnipeButton) {
        existingSnipeButton.remove();
      } else {
        return;
      }
    }

    const tokenMint = card
      .querySelector(".js-copy-to-clipboard")
      ?.getAttribute("data-address");
    if (!tokenMint) return;

    const poolUrl = card.querySelector("a[href*='/lp/']")?.href;
    if (!poolUrl) return;
    const poolId = poolUrl.split("lp/")[1].split("?")[0];
    if (!poolId) return;

    let actionArea = card.querySelector("button");
    if (isMigrating) {
      actionArea = Array.from(card.querySelectorAll("span")).find(
        (span) => span.textContent === "Migrating...",
      );
    }
    if (!actionArea) return;

    if (isMigrating) {
      addMemescopeSnipingButton(actionArea, tokenMint);
    } else {
      addMemescopeBuyButton(actionArea, poolId, quickBuyAmount);
    }
  } catch (error) {
    console.error("Failed to add Bloom button:", error);
  }
}

function addMemescopeSnipingButton(actionArea, tokenMint) {
  const buttonText = "🌸 Snipe";

  const bloomButton = document.createElement("button");
  bloomButton.innerHTML = buttonText;
  bloomButton.type = "button";
  bloomButton.classList.add(
    "bloom-snipe-qt-btn",
    "c-btn",
    "c-btn--lt",
    "u-px-xs",
  );
  bloomButton.style.bottom = "3px";
  bloomButton.style.right = "6px";
  bloomButton.style.position = "relative";
  bloomButton.style.zIndex = "1000";
  bloomButton.onclick = async function (event) {
    event.preventDefault();
    event.stopPropagation();

    const token =
      (await chrome.storage.local.get("bloom.token"))?.["bloom.token"] || "";
    if (!token) {
      return chrome.runtime.sendMessage({
        message: "openTab",
        url: `https://t.me/BloomSolana_bot?start=ref_QT_sniper_${tokenMint}`,
      });
    }

    const order = await createOrder(tokenMint, true, 0, token, "buy");
    if (order?.status === "success") {
      showToast("Order placed successfully!");
    } else {
      showToast("Failed placing order!");
    }
  };
  insertBefore(actionArea, bloomButton);
}

function addMemescopeBuyButton(actionArea, poolId, quickBuyAmount) {
  const bloomButton = actionArea.cloneNode(true);
  bloomButton.style.bottom = "3px";
  bloomButton.style.right = "6px";
  bloomButton.style.position = "relative";
  bloomButton.style.zIndex = "1000";
  bloomButton.classList.add("bloom-buy-qt-btn");

  const firstSpan = bloomButton.querySelector("span");
  if (!firstSpan) return;
  firstSpan.textContent = "🌸";
  firstSpan.classList.remove("c-icon--flash");
  firstSpan.style.backgroundColor = "transparent";
  firstSpan.style.marginRight = "4px";

  const allTextEllipsis = Array.from(
    bloomButton.querySelectorAll("div.text-ellipsis"),
  );
  const lastSpan = allTextEllipsis[allTextEllipsis.length - 1];
  if (!lastSpan) return;

  lastSpan.textContent = `${quickBuyAmount}`;

  bloomButton.onclick = async function (event) {
    event.preventDefault();
    event.stopPropagation();

    const token =
      (await chrome.storage.local.get("bloom.token"))?.["bloom.token"] || "";
    if (!token) {
      return chrome.runtime.sendMessage({
        message: "openTab",
        url: `https://t.me/BloomSolana_bot?start=ref_QT_ca_${poolId}`,
      });
    }

    const order = await createOrder(
      poolId,
      false,
      quickBuyAmount,
      token,
      "buy",
    );
    if (order?.status === "success") {
      showToast("Order placed successfully!");
    } else {
      showToast("Failed placing order!");
    }
  };

  insertBefore(actionArea, bloomButton);
}

async function addDiscoverTrendingBuyButton(card, quickBuyAmount) {
  try {
    const existingBuyButton = card.querySelector(".bloom-buy-qt-btn");

    if (existingBuyButton) {
      existingBuyButton.remove();
    }

    const actionArea = card.querySelector("button");
    if (!actionArea) return;

    const poolId = card.href.split("lp/")[1].split("?")[0];
    if (!poolId) return;

    const buttonClass = "bloom-buy-qt-btn";

    const bloomButton = actionArea.cloneNode(true);
    const firstSpan = bloomButton.querySelector("span");
    if (!firstSpan) return;

    firstSpan.textContent = "🌸";
    firstSpan.classList.remove("c-icon--flash");
    firstSpan.style.backgroundColor = "transparent";

    const allTextEllipsis = Array.from(
      bloomButton.querySelectorAll("div.text-ellipsis"),
    );
    const lastSpan = allTextEllipsis[allTextEllipsis.length - 1];
    if (!lastSpan) return;

    lastSpan.textContent = `${quickBuyAmount}`;

    bloomButton.type = "button";
    bloomButton.classList.add(buttonClass);
    bloomButton.style.margin = "4px 0px";
    bloomButton.style.zIndex = "1000";
    actionArea.style.marginTop = "-6px";

    bloomButton.onclick = async function (event) {
      event.preventDefault();
      event.stopPropagation();

      const token =
        (await chrome.storage.local.get("bloom.token"))?.["bloom.token"] || "";
      if (!token) {
        return chrome.runtime.sendMessage({
          message: "openTab",
          url: `https://t.me/BloomSolana_bot?start=ref_QT_ca_${poolId}`,
        });
      }

      const order = await createOrder(
        poolId,
        false,
        quickBuyAmount,
        token,
        "buy",
      );
      if (order?.status === "success") {
        showToast("Order placed successfully!");
      } else {
        showToast("Failed placing order!");
      }
    };

    insertAfter(actionArea, bloomButton);
  } catch (error) {
    console.error("Failed to add Bloom button:", error);
  }
}
